import 'package:flutter/material.dart';

///tipos de animaciones
enum AnimationType { normal, fadeIn }

///Main Class, [context] es el BuildContext de la aplicación en ese momento
///[child] es el widget a navegar, [animation] es el tipo de animación
class RouteTransitions {
  final BuildContext context;
  final Widget child;
  final AnimationType animation;
  final Duration duration;
  final bool replacement;

  RouteTransitions(
      {@required this.context,
      @required this.child,
      this.replacement = false,
      this.animation = AnimationType.normal,
      this.duration = const Duration(milliseconds: 300)
      //
      }) {
    switch (this.animation) {
      case AnimationType.normal:
        this._normalTransition();
        break;
      case AnimationType.fadeIn:
        this._fadeInTransition();
        break;
    }
  }

  ///otros tipos de navegación
  void _pushPage(Route route) => Navigator.push(context, route);

  ///otros tipos de navegación borrando el stack
  void _pushReplacementPage(Route route) =>
      Navigator.pushReplacement(context, route);

  ///navegacion normal
  void _normalTransition() {
    final route = MaterialPageRoute(builder: (_) => this.child);

    ///navegacion
    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }

  void _fadeInTransition() {
    //transición con animacion
    final route = PageRouteBuilder(
        pageBuilder: (_, __, ___) => this.child,
        transitionDuration: this.duration,
        transitionsBuilder: (_, animation, __, child) {
          //
          return FadeTransition(
            child: child,
            opacity: Tween<double>(begin: 0.0, end: 1.0).animate(
              CurvedAnimation(parent: animation, curve: Curves.easeOut),
            ),
          );
        });

    ///navegacion tomando una decisión por inicialización de la clase
    (this.replacement)
        ? this._pushReplacementPage(route)
        : this._pushPage(route);
  }
}
